package ru.rtlabs.smev3.crypto.examples;

import ru.voskhod.crypto.DigitalSignatureFactory;
import ru.voskhod.crypto.DigitalSignatureProcessor;
import ru.voskhod.crypto.XMLTransformHelper;
import ru.voskhod.crypto.exceptions.SignatureValidationException;
import ru.voskhod.crypto.impl.ValidationResult;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.nio.charset.StandardCharsets;
import java.util.List;
import java.util.stream.Collectors;

/**
 * Проверка XML подписи.
 */
public class CheckXMLSignature {

    public static void main(String[] args) throws IOException {
        String source;
        try (InputStream inputStream = CheckXMLSignature.class.getClassLoader().getResourceAsStream("SignedEnvelope.xml")) {
            source = new BufferedReader(new InputStreamReader(inputStream, StandardCharsets.UTF_8))
                    .lines()
                    .collect(Collectors.joining("\n"));
        }
        DigitalSignatureFactory.init(); //Инициализация крипто библиотеки
        DigitalSignatureProcessor dsp = DigitalSignatureFactory.getDigitalSignatureProcessor(); //получение процессора подписи

        try {
            List<ValidationResult> validationResults = dsp.validateXMLDSigEnvelopedAllSignature(
                    XMLTransformHelper.buildDocumentFromString(source).getDocumentElement());
            for (ValidationResult result : validationResults) {
                if (result.isValid()) {
                    System.out.println("Подпись " + result.getSignatureId() + " верна: Подписано " + result.getCertificate().getSubjectX500Principal().getName());
                    //Не забудьте проверить сертификат (отзыв)
                } else {
                    //Проверка завершилась неудачей
                    System.out.println(result.getError());
                }
            }
        } catch (SignatureValidationException e) {
            //Ошибка проверки подписи
            System.out.println(e.getMessage());
        }
    }

}
