package ru.rtlabs.smev3.crypto.examples;

import ru.voskhod.crypto.DigitalSignatureFactory;
import ru.voskhod.crypto.DigitalSignatureProcessor;
import ru.voskhod.crypto.KeyStoreWrapper;
import ru.voskhod.crypto.exceptions.SignatureProcessingException;

import java.io.IOException;
import java.io.InputStream;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.UnrecoverableKeyException;
import java.security.cert.CertificateException;
import java.util.Base64;

/**
 * Подпись вложения.
 */
public class SignAttachment {

    public static final String CONTAINER_ALIAS = "testContainer256"; //CHANGE ME: Алиас контейнера для подписания
    public static final String CONTAINER_PASSWORD = "1234567890"; //CHANGE ME: Пароль контейнера для подписания

    public static void main(String[] args) throws UnrecoverableKeyException, KeyStoreException, NoSuchAlgorithmException,
            CertificateException, SignatureProcessingException, IOException {
        try (InputStream source = SignAttachment.class.getClassLoader().getResourceAsStream("TestFile.txt")) {
            DigitalSignatureFactory.init(); //Инициализация крипто библиотеки
            DigitalSignatureProcessor dsp = DigitalSignatureFactory.getDigitalSignatureProcessor(); //получение процессора подписи
            KeyStoreWrapper ksw = DigitalSignatureFactory.getKeyStoreWrapper(); //Получение хранилища по умолчанию - HDImageStore

            final byte[] attachmentSignature = dsp.signPKCS7Detached(source, ksw.getPrivateKey(CONTAINER_ALIAS, CONTAINER_PASSWORD.toCharArray()),
                    ksw.getX509Certificate(CONTAINER_ALIAS)); //Подписание
            System.out.println("pkcs7Signature: " + Base64.getEncoder().encodeToString(attachmentSignature));
        }
    }

}
