package ru.rtlabs.smev3.crypto.examples;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import ru.voskhod.crypto.DigitalSignatureFactory;
import ru.voskhod.crypto.DigitalSignatureProcessor;
import ru.voskhod.crypto.KeyStoreWrapper;
import ru.voskhod.crypto.XMLTransformHelper;
import ru.voskhod.crypto.exceptions.SignatureProcessingException;

import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.UnrecoverableKeyException;
import java.security.cert.CertificateException;

/**
 * Подпись конверта.
 */
public class SignXml {

    public static final String MESSAGE_EXCHANGE_TYPES_1_2 = "urn://x-artefacts-smev-gov-ru/services/message-exchange/types/1.2";
    public static final String CONTAINER_ALIAS = "testContainer256"; //CHANGE ME: Алиас контейнера для подписания
    public static final String CONTAINER_PASSWORD = "1234567890"; //CHANGE ME: Пароль контейнера для подписания

    public static void main(String[] args) throws UnrecoverableKeyException, KeyStoreException, NoSuchAlgorithmException,
            CertificateException, SignatureProcessingException {
        String source = "<soapenv:Envelope xmlns:ns=\"urn://x-artefacts-smev-gov-ru/services/message-exchange/types/1.2\" xmlns:ns1=\"urn://x-artefacts-smev-gov-ru/services/message-exchange/types/basic/1.2\" xmlns:soapenv=\"http://schemas.xmlsoap.org/soap/envelope/\">\n" +
                "\t<soapenv:Header/>\n" +
                "\t<soapenv:Body>\n" +
                "\t\t<ns:SendRequestRequest>\n" +
                "\t\t\t<ns:SenderProvidedRequestData Id=\"SIGNED_BY_CALLER\">\n" +
                "\t\t\t\t<ns:MessageID>f1a95957-ba1c-11eb-9491-00059a2c7a00</ns:MessageID>\n" +
                "\t\t\t\t<ns:ReferenceMessageID>f1a95957-ba1c-11eb-9491-00059a8c7a00</ns:ReferenceMessageID>\n" +
                "\t\t\t\t<ns:EOL>2021-05-23T13:11:44.120+03:00</ns:EOL>\n" +
                "\t\t\t\t<ns1:MessagePrimaryContent>\n" +
                "\t\t\t\t\t<test:TestExample xmlns:test=\"urn://test.smev.ru\">\n" +
                "\t\t\t\t\t\t<test:TestData>data</test:TestData>\n" +
                "\t\t\t\t\t</test:TestExample>\n" +
                "\t\t\t\t</ns1:MessagePrimaryContent>\n" +
                "\t\t\t</ns:SenderProvidedRequestData>\n" +
                "\t\t\t<ns:CallerInformationSystemSignature/>\n" +
                "\t\t</ns:SendRequestRequest>\n" +
                "\t</soapenv:Body>\n" +
                "</soapenv:Envelope>";

        DigitalSignatureFactory.init(); //Инициализация крипто библиотеки
        DigitalSignatureProcessor dsp = DigitalSignatureFactory.getDigitalSignatureProcessor(); //получение процессора подписи
        KeyStoreWrapper ksw = DigitalSignatureFactory.getKeyStoreWrapper(); //Получение хранилища по умолчанию - HDImageStore

        Document document = XMLTransformHelper.buildDocumentFromString(source); //Получаем DOM
        final Element elementToSign = (Element) document.getElementsByTagNameNS(MESSAGE_EXCHANGE_TYPES_1_2,
                "SenderProvidedRequestData").item(0); //Элемент дла подписания
        Element signature = dsp.signXMLDSigDetached(elementToSign, null,
                ksw.getPrivateKey(CONTAINER_ALIAS, CONTAINER_PASSWORD.toCharArray()), ksw.getX509Certificate(CONTAINER_ALIAS)); //Подписание
        System.out.println("Подпись:\n" + XMLTransformHelper.elementToString(signature, true));

        final Node elementToPlaceSignature = document.getElementsByTagNameNS(MESSAGE_EXCHANGE_TYPES_1_2,
                "CallerInformationSystemSignature").item(0);
        elementToPlaceSignature.appendChild(signature); //Вставка подписи в нужный элемент

        System.out.println("Подписанный документ:\n" + XMLTransformHelper.elementToString(document.getDocumentElement(), false));
    }

}
